/*
 * Configuration Wizard for Kerio Mailserver
 * Copyright (C) Kerio Technologies, s.r.o.
 * 
 * This program (Configuration Wizard for Kerio Mailserver) is free 
 * software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * To compile this prohram, please install newt-devel and slang-devel and run:
 *		   gcc wizard.cpp -o wizard -lnewt -lslang -lstdc++
 */
#include <newt.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <netdb.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/utsname.h> 

int mcAfee = 0;
char *sFileName;
char *uFileName;

#define CFGWIZ_VERSION		"6.3.0 build 1"

#define CFG_NOEXISTS		0x00000000
#define CFG_MAILSERVER	0x00000001
#define CFG_USER				0x00000002

#define STATE_END				0
#define STATE_WELCOME		1
#define STATE_DOMAIN		2
#define STATE_ADMIN			3
#define STATE_STORE			4
#define STATE_FINISH		5


bool createAdminDir(const char* dirName) 
{
    if (mkdir(dirName, 0700) != 0)
        return false;

    if (chmod(dirName, 0700) != 0)
        return false;

    return true;
}

void getHostbyname(char *hostname, int hostlen)
{
	int result = 0;
    struct utsname un;
	if (uname(&un) == 0) {
		result = 1;
        
		struct hostent * host = gethostbyname(un.nodename);
		if (host != NULL)
			strncpy(hostname, host->h_name, hostlen);
		else
			result = 0;
	}

	if (result == 0)
		strncpy(hostname, "unspecified.host", hostlen);
}

int showErrCfg(unsigned int flags)
{
	int width = 68;
	
	if(flags == CFG_NOEXISTS)
		return 1;

	newtComponent yesButton, noButton, errMsg1, errMsg2, errMsg3, form;
	newtComponent returned;

	bool cfgmail = ((flags & CFG_MAILSERVER) != 0);
	bool cfguser = ((flags & CFG_USER) != 0);

	if(cfgmail && cfguser) {
		errMsg1 = newtLabel(1, 1, "The configuration files 'mailserver.cfg' and 'users.cfg' already exist.");
		width = 72;
	}
	else if(cfguser)
		errMsg1 = newtLabel(1, 1, "The configuration file 'users.cfg' already exists.");
	else if(cfgmail)
		errMsg1 = newtLabel(1, 1, "The configuration file 'mailserver.cfg' already exists.");
	else
		return 1;

	newtCenteredWindow(width, 8, "Warning");
	errMsg2 = newtLabel(1, 3, "Do you want to continue to run the Configuration wizard?");
	errMsg3 = newtLabel(1, 4, "If you choose 'Yes', the old configuration files will be replaced.");

	noButton = newtCompactButton((width / 2 - 10), 6, "No");
	yesButton = newtCompactButton((width / 2 + 5), 6, "Yes");

	form = newtForm(NULL, NULL, 0);
	newtFormAddComponents(form, errMsg1, errMsg2, errMsg3, noButton, yesButton, NULL);

	returned = newtRunForm(form);
	newtFormDestroy(form);

	newtPopWindow();

	return (returned == yesButton);
}

void showErrDomain()
{
	newtComponent okButton, errMsg, form;

	newtCenteredWindow(26, 5, "Error");
	errMsg = newtLabel(1, 1, "Domain name must be set!");
	okButton = newtCompactButton(11, 3, "OK");

	form = newtForm(NULL, NULL, 0);
	newtFormAddComponents(form, errMsg, okButton, NULL);

	newtRunForm(form);
	newtFormDestroy(form);

	newtPopWindow();
}

void showErrName(void)
{
	newtComponent okButton, errMsg, form;

	newtCenteredWindow(34, 5, "Error");
	errMsg = newtLabel(1, 1, "Administrator name must be set!");
	okButton = newtCompactButton(15, 3, "OK");

	form = newtForm(NULL, NULL, 0);
	newtFormAddComponents(form, errMsg, okButton, NULL);

	newtRunForm(form);
	newtFormDestroy(form);

	newtPopWindow();
}

int showErrLength(void)
{
	newtComponent yesButton, noButton, errMsg1, errMsg2, errMsg3, form;
	newtComponent returned;

	newtCenteredWindow(34, 8, "Error");
	errMsg1 = newtLabel(1, 1, "Administrator password should");
	errMsg2 = newtLabel(1, 2, "be at least six characters long!");
	errMsg3 = newtLabel(1, 4, "Do you want to continue?");
	noButton = newtCompactButton(6, 6, "No");
	yesButton = newtCompactButton(20, 6, "Yes");

	form = newtForm(NULL, NULL, 0);
	newtFormAddComponents(form, errMsg1, errMsg2, errMsg3, noButton, yesButton, NULL);

	returned = newtRunForm(form);
	newtFormDestroy(form);

	newtPopWindow();

	return (returned == yesButton);
}

void showErrMatch(void)
{
	newtComponent okButton, errMsg, form;

	newtCenteredWindow(30, 5, "Error");
	errMsg = newtLabel(1, 1, "Given passwords don't match!");
	okButton = newtCompactButton(13, 3, "OK");

	form = newtForm(NULL, NULL, 0);
	newtFormAddComponents(form, errMsg, okButton, NULL);

	newtRunForm(form);
	newtFormDestroy(form);

	newtPopWindow();
}

void showErrDir(void)
{
	newtComponent okButton, errMsg, form;

	newtCenteredWindow(30, 5, "Error");
	errMsg = newtLabel(1, 1, "Store directory must be set!");
	okButton = newtCompactButton(13, 3, "OK");

	form = newtForm(NULL, NULL, 0);
	newtFormAddComponents(form, errMsg, okButton, NULL);

	newtRunForm(form);
	newtFormDestroy(form);

	newtPopWindow();
}

int welcome()
{
	newtComponent nextButton, cancelButton;
	newtComponent form, label1, label2, backLabel;
	newtComponent returned;

	newtCenteredWindow(41, 6, "Welcome");

	label1 = newtLabel(1, 1, "This wizard helps you to enter the most");
	label2 = newtLabel(1, 2, "essential data for Kerio MailServer.");
	backLabel = newtLabel(1, 4, "<Back>");
	nextButton = newtCompactButton(10, 4, "Next");
	cancelButton = newtCompactButton(30, 4, "Cancel");

	form = newtForm(NULL, NULL, 0);
	newtFormAddComponents(form, label1, label2, backLabel, nextButton, cancelButton, NULL);

	returned = newtRunForm(form);

	newtFormDestroy(form);
	newtPopWindow();

	if(returned == cancelButton)
		return 0;
	else
		return 2;
}

int domain(char **domainName, char** hostname)
{
	newtComponent nextButton, backButton, cancelButton;
	newtComponent form, label1, label2, label3;
	newtComponent entryBox;
	newtComponent returned;
	newtComponent lblHostname1;
	newtComponent lblHostname2;
	newtComponent lblHostname3;
	newtComponent edHostname;
	char *value;
	char *valueHostname;
	int retVal;
	int ok = 0;

	char defHostname[256];
	getHostbyname(defHostname, sizeof(defHostname));

	newtCenteredWindow(50, 17, "Mail Domain & Internet Hostname");

	label1 = newtLabel(1, 1, "Please enter a name of a primary mail domain");
	label2 = newtLabel(1, 2, "that will be created now.");
	entryBox = newtEntry(1, 4, (*domainName ? *domainName : "localhost"), 45, (const char**) &value, NEWT_FLAG_SCROLL | NEWT_FLAG_RETURNEXIT);
	label3 = newtLabel(1, 5, "(For example: example.com)");

	lblHostname1 = newtLabel(1, 8, "Please enter a domain name that resolves to the");
	lblHostname2 = newtLabel(1, 9, "IP address of the Kerio MailServer computer.");
	edHostname = newtEntry(1, 11, (*hostname ? *hostname : defHostname), 45, (const char**) &valueHostname, NEWT_FLAG_SCROLL | NEWT_FLAG_RETURNEXIT);
	lblHostname3 = newtLabel(1, 12, "(For example: mail.example.com)");

	backButton = newtCompactButton(1, 15, "Back");
	nextButton = newtCompactButton(10, 15, "Next");
	cancelButton = newtCompactButton(39, 15, "Cancel");

	form = newtForm(NULL, NULL, 0);
	newtFormAddComponents(form, label1, label2, entryBox, label3, lblHostname1, lblHostname2, edHostname, lblHostname3,
						backButton, nextButton, cancelButton, NULL);

	while (!ok) {
		returned = newtRunForm(form);
		if ((returned != cancelButton) && (returned != backButton)) {
			if (!value[0]) showErrDomain();
			else ok = 1;
		}
		else ok = 1;
	}

	if (returned == cancelButton) retVal = STATE_END;
	else if (returned == backButton) retVal = STATE_WELCOME;
	else {
		if (*domainName != NULL) 
			free(*domainName);

		*domainName = strdup(value);

		if (*hostname != NULL) 
			free(*hostname);

		*hostname = strdup(valueHostname);

		retVal = STATE_ADMIN;
	}

	newtFormDestroy(form);
	newtPopWindow();

	return retVal;
}

int admin(char **name, char **password)
{
	newtComponent nextButton, backButton, cancelButton;
	newtComponent form, label1, label2, label3, label4, label5, label6, label7, label8;
	newtComponent entryName, entryPassword1, entryPassword2;
	newtComponent labelName, labelPassword1, labelPassword2;
	newtComponent returned;
	char *value1 = NULL, *value2 = NULL, *value3 = NULL;
	int ok = 0;
	int retVal;

	newtCenteredWindow(46, 19, "Administrative Account");

	label1 = newtLabel(1, 1, "Enter the name and password of an user");
	label2 = newtLabel(1, 2, "who will have full administration rights");
	label3 = newtLabel(1, 3, "for your server. The password should be");
	label4 = newtLabel(1, 4, "at least six characters long.");
	label5 = newtLabel(1, 6, "After the installation, the administrator");
	label6 = newtLabel(1, 7, "can launch the Kerio Administration Console,");
	label7 = newtLabel(1, 8, "login to the server using his/her");
	label8 = newtLabel(1, 9, "username and start configuring the server.");
	backButton = newtCompactButton(1, 17, "Back");
	nextButton = newtCompactButton(10, 17, "Next");
	cancelButton = newtCompactButton(30, 17, "Cancel");
	labelName = newtLabel(1, 13, "Username:");
	entryName = newtEntry(20, 13, (*name == NULL)?"Admin":*name, 20, (const char**) &value1, NEWT_FLAG_SCROLL);
	labelPassword1 = newtLabel(1, 14, "Password:");
	entryPassword1 = newtEntry(20, 14, *password, 20, (const char**) &value2, NEWT_FLAG_SCROLL | NEWT_FLAG_HIDDEN);
	labelPassword2 = newtLabel(1, 15, "Confirm password:");
	entryPassword2 = newtEntry(20, 15, *password, 20,(const char**) &value3, NEWT_FLAG_SCROLL | NEWT_FLAG_RETURNEXIT | NEWT_FLAG_HIDDEN);

	form = newtForm(NULL, NULL, 0);
	newtFormAddComponents(form, label1, label2, label3, label4, label5,
						  label6, label7, label8,
						  labelName, entryName,
						  labelPassword1, entryPassword1,
						  labelPassword2, entryPassword2,
						  backButton, nextButton, cancelButton, NULL);

	while (!ok) {
		returned = newtRunForm(form);
		if ((returned != cancelButton) && (returned != backButton)) {
			if (!value1[0]) {
				showErrName();
				newtEntrySet(entryName, "Admin", 1);
			}
			else if (strcmp(value2, value3)) {
				showErrMatch();
			}
			else if (strlen(value2) < 6) {
				if (showErrLength())
					ok = 1;
			}
			else ok = 1;
		}
		else ok = 1;
	}

	if (returned == cancelButton) retVal = STATE_END;
	else if (returned == backButton) retVal = STATE_DOMAIN;
	else {
		if (*name != NULL) free(*name);
		*name = strdup(value1);

		if (*password != NULL) free(*password);
		*password = strdup(value2);

		retVal = STATE_STORE;
	}

	newtFormDestroy(form);
	newtPopWindow();

	return retVal;
}

int store(char **storePath)
{
	newtComponent finishButton, backButton, cancelButton;
	newtComponent form, label1, label2, label3, label4, label5, label6, label7, label8;
	newtComponent entryBox;
	newtComponent returned;
	char *value;
	int retVal;
	int ok = 0;

	newtCenteredWindow(44, 15, "Message Store Directory");

	label1 = newtLabel(1, 1, "Kerio MailServer will store all messages");
	label2 = newtLabel(1, 2, "in a selected directory. The volume on");
	label3 = newtLabel(1, 3, "which the directory will be located should");
	label4 = newtLabel(1, 4, "have enough space to hold messages.");
	label5 = newtLabel(1, 6, "The space required depends on many factors");
	label6 = newtLabel(1, 7, "like number of users, expected mail");
	label7 = newtLabel(1, 8, "traffic or whether e-mail clients are");
	label8 = newtLabel(1, 9, "configured to keep messages on server.");
	backButton = newtCompactButton(1, 13, "Back");
	finishButton = newtCompactButton(10, 13, "Finish");
	cancelButton = newtCompactButton(30, 13, "Cancel");
	entryBox = newtEntry(1, 11, (*storePath == NULL)?"/opt/kerio/mailserver/store":*storePath,
						 42,(const char**) &value, NEWT_FLAG_SCROLL | NEWT_FLAG_RETURNEXIT);

	form = newtForm(NULL, NULL, 0);
	newtFormAddComponents(form, label1, label2, label3, label4, label5, label6, label7, label8,
						  entryBox, backButton, finishButton, cancelButton, NULL);

	while (!ok) {
		returned = newtRunForm(form);
		if ((returned != cancelButton) && (returned != backButton)) {
			if (!value[0]) {
				showErrDir();
				newtEntrySet(entryBox, "/opt/kerio/mailserver/store", 1);
			}
			else ok = 1;
		}
		else ok = 1;
	}

	if (returned == cancelButton) retVal = STATE_END;
	else if (returned == backButton) retVal = STATE_ADMIN;
	else {
		if (*storePath != NULL) free(*storePath);
		*storePath = strdup(value);

		retVal = STATE_FINISH;
	}

	newtFormDestroy(form);
	newtPopWindow();

	return retVal;
}

void writeCfg(const char *mailDomain, const char* hostname, const char *storePath, const char* adminName)
{
	FILE *f = fopen(sFileName, "wb");

	if (f == NULL) {
		newtPushHelpLine("Unable to open file mailserver.cfg");
		newtBell();
		newtRefresh();
		sleep(2);
		newtPopHelpLine();
		return;
	}

	char* kerberosRealm = new char[strlen(mailDomain) + 1];
	strcpy(kerberosRealm, mailDomain);

	int index = 0;
	unsigned char myc;

	while((myc = kerberosRealm[index]) != 0) {
		if(islower(myc))
			kerberosRealm[index] = toupper(myc);

		++index;
	}

	fprintf(f, "<config>\r\n");
	fprintf(f, "<list name=\"Domain\">\r\n");
	fprintf(f, "  <listitem>\r\n");
	fprintf(f, "    <variable name=\"Domain\">%s</variable>\r\n", mailDomain);
	fprintf(f, "    <variable name=\"Type\">0</variable>\r\n");
	fprintf(f, "    <variable name=\"Alias\"></variable>\r\n");
	fprintf(f, "    <variable name=\"StoreType\">0</variable>\r\n");
	fprintf(f, "    <variable name=\"BindIp\">0.0.0.0</variable>\r\n");
	fprintf(f, "    <variable name=\"FwHost\"></variable>\r\n");
	fprintf(f, "    <variable name=\"FwPort\">25</variable>\r\n");
	fprintf(f, "    <variable name=\"FwOffline\">0</variable>\r\n");
	fprintf(f, "    <variable name=\"NtAuthDomain\"></variable>\r\n");
	fprintf(f, "    <variable name=\"PamService\"></variable>\r\n");

	fprintf(f, "    <variable name=\"KerberosRealm\">%s</variable>\r\n", kerberosRealm);
	delete[] kerberosRealm;

	fprintf(f, "    <variable name=\"UseLdap\">0</variable>\r\n");
	fprintf(f, "    <variable name=\"Description\"></variable>\r\n");
	fprintf(f, "  </listitem>\r\n");
	fprintf(f, "</list>\r\n");
	fprintf(f, "<list name=\"AttachFilter\">\r\n");
	fprintf(f, "  <listitem>\r\n");
	fprintf(f, "    <variable name=\"Type\">0</variable>\r\n");
	fprintf(f, "    <variable name=\"Pattern\">*.exe</variable>\r\n");
	fprintf(f, "    <variable name=\"Action\">1</variable>\r\n");
	fprintf(f, "    <variable name=\"Order\">0</variable>\r\n");
	fprintf(f, "    <variable name=\"Description\">EXE files</variable>\r\n");
	fprintf(f, "    <variable name=\"Enabled\">0</variable>\r\n");
	fprintf(f, "  </listitem>\r\n");
	fprintf(f, "  <listitem>\r\n");
	fprintf(f, "    <variable name=\"Type\">0</variable>\r\n");
	fprintf(f, "    <variable name=\"Pattern\">*.com</variable>\r\n");
	fprintf(f, "    <variable name=\"Action\">1</variable>\r\n");
	fprintf(f, "    <variable name=\"Order\">1</variable>\r\n");
	fprintf(f, "    <variable name=\"Description\">COM files</variable>\r\n");
	fprintf(f, "    <variable name=\"Enabled\">1</variable>\r\n");
	fprintf(f, "  </listitem>\r\n");
	fprintf(f, "  <listitem>\r\n");
	fprintf(f, "    <variable name=\"Type\">0</variable>\r\n");
	fprintf(f, "    <variable name=\"Pattern\">*.scr</variable>\r\n");
	fprintf(f, "    <variable name=\"Action\">1</variable>\r\n");
	fprintf(f, "    <variable name=\"Order\">2</variable>\r\n");
	fprintf(f, "    <variable name=\"Description\">Screenshots files</variable>\r\n");
	fprintf(f, "    <variable name=\"Enabled\">1</variable>\r\n");
	fprintf(f, "  </listitem>\r\n");
	fprintf(f, "  <listitem>\r\n");
	fprintf(f, "    <variable name=\"Type\">0</variable>\r\n");
	fprintf(f, "    <variable name=\"Pattern\">*.bat</variable>\r\n");
	fprintf(f, "    <variable name=\"Action\">1</variable>\r\n");
	fprintf(f, "    <variable name=\"Order\">3</variable>\r\n");
	fprintf(f, "    <variable name=\"Description\">BAT files</variable>\r\n");
	fprintf(f, "    <variable name=\"Enabled\">1</variable>\r\n");
	fprintf(f, "  </listitem>\r\n");
	fprintf(f, "  <listitem>\r\n");
	fprintf(f, "    <variable name=\"Type\">0</variable>\r\n");
	fprintf(f, "    <variable name=\"Pattern\">*.vbs</variable>\r\n");
	fprintf(f, "    <variable name=\"Action\">1</variable>\r\n");
	fprintf(f, "    <variable name=\"Order\">4</variable>\r\n");
	fprintf(f, "    <variable name=\"Description\">Visual Basic scripts</variable>\r\n");
	fprintf(f, "    <variable name=\"Enabled\">1</variable>\r\n");
	fprintf(f, "  </listitem>\r\n");
	fprintf(f, "  <listitem>\r\n");
	fprintf(f, "    <variable name=\"Type\">0</variable>\r\n");
	fprintf(f, "    <variable name=\"Pattern\">*.{*}*</variable>\r\n");
	fprintf(f, "    <variable name=\"Action\">1</variable>\r\n");
	fprintf(f, "    <variable name=\"Order\">5</variable>\r\n");
	fprintf(f, "    <variable name=\"Description\">CLSID extension vulnerability</variable>\r\n");
	fprintf(f, "    <variable name=\"Enabled\">1</variable>\r\n");
	fprintf(f, "  </listitem>\r\n");
	fprintf(f, "  <listitem>\r\n");
	fprintf(f, "    <variable name=\"Type\">0</variable>\r\n");
	fprintf(f, "    <variable name=\"Pattern\">*.doc.*</variable>\r\n");
	fprintf(f, "    <variable name=\"Action\">1</variable>\r\n");
	fprintf(f, "    <variable name=\"Order\">6</variable>\r\n");
	fprintf(f, "    <variable name=\"Description\">Double extensions</variable>\r\n");
	fprintf(f, "    <variable name=\"Enabled\">1</variable>\r\n");
	fprintf(f, "  </listitem>\r\n");
	fprintf(f, "  <listitem>\r\n");
	fprintf(f, "    <variable name=\"Type\">0</variable>\r\n");
	fprintf(f, "    <variable name=\"Pattern\">*.xls.*</variable>\r\n");
	fprintf(f, "    <variable name=\"Action\">1</variable>\r\n");
	fprintf(f, "    <variable name=\"Order\">7</variable>\r\n");
	fprintf(f, "    <variable name=\"Description\">Double extensions</variable>\r\n");
	fprintf(f, "    <variable name=\"Enabled\">1</variable>\r\n");
	fprintf(f, "  </listitem>\r\n");
	fprintf(f, "  <listitem>\r\n");
	fprintf(f, "    <variable name=\"Type\">1</variable>\r\n");
	fprintf(f, "    <variable name=\"Pattern\">audio/x-wav</variable>\r\n");
	fprintf(f, "    <variable name=\"Action\">1</variable>\r\n");
	fprintf(f, "    <variable name=\"Order\">8</variable>\r\n");
	fprintf(f, "    <variable name=\"Description\">MSIE vulnerability</variable>\r\n");
	fprintf(f, "    <variable name=\"Enabled\">0</variable>\r\n");
	fprintf(f, "  </listitem>\r\n");
	fprintf(f, "  <listitem>\r\n");
	fprintf(f, "    <variable name=\"Type\">1</variable>\r\n");
	fprintf(f, "    <variable name=\"Pattern\">audio/x-midi</variable>\r\n");
	fprintf(f, "    <variable name=\"Action\">1</variable>\r\n");
	fprintf(f, "    <variable name=\"Order\">9</variable>\r\n");
	fprintf(f, "    <variable name=\"Description\">MSIE vulnerability</variable>\r\n");
	fprintf(f, "    <variable name=\"Enabled\">0</variable>\r\n");
	fprintf(f, "  </listitem>\r\n");
	fprintf(f, "</list>\r\n");
	fprintf(f, "\r\n");
	fprintf(f, "<table name=\"Misc\">\r\n");
	fprintf(f, "  <variable name=\"InternetHostname\">%s</variable>\r\n", hostname);
	fprintf(f, "  <variable name=\"PublicAdmin\">%s@%s</variable>\r\n", adminName, mailDomain);
	fprintf(f, "</table>\r\n");
	fprintf(f, "\r\n");
	fprintf(f, "<table name=\"Backup\">\r\n");
	fprintf(f, "  <variable name=\"Admin\">%s@%s</variable>\r\n", adminName, mailDomain);
	fprintf(f, "</table>\r\n");
	fprintf(f, "\r\n");
	fprintf(f, "<table name=\"Directories\">\r\n");
	fprintf(f, "  <variable name=\"StoreDir\">%s</variable>\r\n", storePath);
	fprintf(f, "</table>\r\n");
	fprintf(f, "\r\n");
	fprintf(f, "<table name=\"Smtp\">\r\n");
	fprintf(f, "  <variable name=\"VerifySenderDomain\">1</variable>\r\n");
	fprintf(f, "</table>\r\n");
	fprintf(f, "\r\n");
	if (mcAfee) {
		fprintf(f, "<table name=\"Antivir\">\r\n");
		fprintf(f, "  <variable name=\"AvirEnabled\">1</variable>\r\n");
		fprintf(f, "  <variable name=\"UsedPlugin\">avir_mcafee</variable>\r\n");
		fprintf(f, "</table>\r\n");
		fprintf(f, "\r\n");
	}
	fprintf(f, "</config>\r\n");

	fclose(f);
	
	chmod(sFileName, 0600);
	chown(sFileName, 0, 0);
}

void writeUsers(const char *adminName, const char *adminPassword, const char *mailDomain)
{
	FILE *f = fopen(uFileName, "wb");
	char *xored;
	int len = strlen(adminPassword);
	int i;
	int newChar;

	xored = (char *)malloc(len*2 +1);

	if (xored == NULL) {
		newtPushHelpLine("Error while storing password to users.cfg");
		newtRefresh();
		newtBell();
		sleep(2);
		newtPopHelpLine();
		return;
	}

	if (f == NULL) {
		newtPushHelpLine("Unable to open file users.cfg");
		newtRefresh();
		sleep(2);
		newtPopHelpLine();
		return;
	}

	for (i = 0; i < len; i++) {
		newChar = adminPassword[i] ^ 0x55;
		sprintf(&(xored[i*2]), "%02x", newChar);
	}
	xored[2*len] = '\0';

	fprintf(f, "<config>\r\n");
	fprintf(f, "<list name=\"user\">\r\n");
	fprintf(f, "  <listitem>\r\n");
	fprintf(f, "    <variable name=\"name\">%s</variable>\r\n", adminName);
	fprintf(f, "    <variable name=\"fullname\">Administrator</variable>\r\n");
	fprintf(f, "    <variable name=\"domain\">%s</variable>\r\n", mailDomain);
	fprintf(f, "    <variable name=\"rights\">3</variable>\r\n");
	fprintf(f, "    <variable name=\"auth_type\">0</variable>\r\n");
	fprintf(f, "    <variable name=\"password\">XOR:%s</variable>\r\n", xored);
	fprintf(f, "  </listitem>\r\n");
	fprintf(f, "</list>\r\n");
	fprintf(f, "</config>\r\n");

	fclose(f);

	chmod(uFileName, 0600);
	chown(uFileName, 0, 0);
}

unsigned int checkConfig(const char *path)
{
	const char *lastSlash;
	char *fileName;
	char usrCfg[] = "users.cfg";
	char srvCfg[] = "mailserver.cfg";
	char plgName[] = "plugins/avir_mcafee.so";
	FILE *f;
	unsigned int cfgRes = CFG_NOEXISTS;

	if (path == NULL) return 0;

	lastSlash = strrchr(path, '/');
	if (lastSlash == NULL) lastSlash = path;
	else lastSlash++;

	fileName = (char *)malloc(strlen(plgName) + (lastSlash - path) + 1);
	if (fileName == NULL) return 0;

	strncpy(fileName, path, lastSlash - path);
	strcat(fileName, srvCfg);

	f = fopen(fileName, "r");
	if (f != NULL) {
		cfgRes |= CFG_MAILSERVER;
		fclose(f);
	}
	sFileName = strdup(fileName);

	fileName[lastSlash - path] = '\0';
	strcat(fileName, usrCfg);

	f = fopen(fileName, "r");
	if (f != NULL) {
		cfgRes |= CFG_USER;
		fclose(f);
	}
	uFileName = strdup(fileName);

	fileName[lastSlash - path] = '\0';
	strcat(fileName, plgName);

	f = fopen(fileName, "r");
	if (f != NULL) {
		fclose(f);
		mcAfee = 1;
	}

	free(fileName);

	return cfgRes;
}

bool createStoreDirectory(const char* directory)
{
	const size_t LEN = strlen(directory) + 1;

	char* dir = new char[LEN];
	snprintf(dir, LEN, "%s", directory);

	char* tempDir = strchr(dir, '/');
	bool res = false;

	if(tempDir) {
		while(tempDir) {
			if(tempDir != dir) {
				tempDir[0] = 0;
				res = createAdminDir(dir);
				tempDir[0] = '/';
			}
		
			tempDir += 1;
			if(tempDir[0]) {
				tempDir = strchr(tempDir, '/');
					if(tempDir == 0)
				res = createAdminDir(dir);
			}
			else
				tempDir = 0;
		}
	}
	else {
		res = createAdminDir(dir);
	}

	chmod(directory, 0700);
	chown(directory, 0, 0);

	delete[] dir;
	return res;
}


int main(int argc, char **argv) 
{
	int state = STATE_WELCOME;
	char *mailDomain = NULL;
	char* hostname = NULL;
	char *adminName = NULL;
	char *adminPassword = NULL;
	char *directory = NULL;
	unsigned int existFlags;
	int index = 0;
	unsigned char myc;

	existFlags = checkConfig(argv[0]);

	newtInit();
	newtCls();

	newtDrawRootText(0, 0, "Kerio MailServer 6 - Configuration Wizard");

	if(argc > 1 && strcmp(argv[1], "-v") == 0)
		newtDrawRootText(0, 1, "Version: " CFGWIZ_VERSION);

	while (state != STATE_END) {
		switch (state) {
		case STATE_WELCOME:
			if (existFlags != CFG_NOEXISTS && !showErrCfg(existFlags)) {
				state = STATE_END;
				break;
			}
			
			state = welcome();
			break;
		case STATE_DOMAIN:
			state = domain(&mailDomain, &hostname);

			break;
		case STATE_ADMIN:
			state = admin(&adminName, &adminPassword);
			break;
		case STATE_STORE:
			state = store(&directory);
			break;
		case STATE_FINISH:

			while((myc = mailDomain[index]) != 0) {
				if(isupper(myc))
				mailDomain[index] = tolower(myc);

				++index;
			}

			newtPushHelpLine("Saving server configuration");
			newtRefresh();
			writeCfg(mailDomain, hostname, directory, adminName);
			newtPushHelpLine("Creating store directory");
			newtRefresh();
			createStoreDirectory(directory);
			newtPushHelpLine("Saving user configuration");
			newtRefresh();
			writeUsers(adminName, adminPassword, mailDomain);
			newtPopHelpLine();
			state = STATE_END;
			break;
		default:
			state = STATE_END;
		}
	}

    newtFinished();
}
